/*
  Copyright (C) 2004 Hideo Bannai (bannai@ims.u-tokyo.ac.jp)
  
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
*/

#include "FastaFile.h"
#include "StringUtils.h"
#include <iostream>
#include <fstream>

void FastaFile::read(const std::string & filename,
					 std::multimap<std::string,FastaEntry> & db){
	std::string line, nm, seq;
	std::ifstream f(filename.c_str(), std::fstream::in);

	std::getline(f, line);   // read first line
	if(!f.is_open()){
		std::cerr << "FastaFile: Error: Failed to open file '"
		          << filename << "'. Aborting." << std::endl;
		exit(1);
	}
	while(!f.eof()){
		// skip until header
		while(line.empty() || (line[0] != '>')){
			if(f.eof()){
				// safely end if we reach eof before finding a header line
				f.close();
				return;
			}
			std::getline(f, line);
		}
		nm = std::string(line, 1);
		StringUtils::strip_d(nm);
		std::getline(f, line);
		while((!line.empty()) && (line[0] != '>')){
			StringUtils::strip_d(line);
			seq.append(line);
			if(f.eof()) break;
			std::getline(f, line);
		}
		FastaEntry fe(nm, seq);
		db.insert(std::pair<std::string,FastaEntry>(nm,fe));
		nm.clear();
		seq.clear();
		if(line.length() > 0) f.clear();
	}
	f.close();
	return;
}
