/*
  Copyright (C) 2004 Hideo Bannai (bannai@ims.u-tokyo.ac.jp)
  
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
*/

#include "NumericDataValue.h"
#include "Scores.h"
#include <iostream>
#include <sstream>

const NumericDataValue NumericDataValue::zero = NumericDataValue(0,0.0);

NumericDataValue::NumericDataValue()
: cnt(0), numeric_val(0.0){
}

NumericDataValue::~NumericDataValue()
{}

NumericDataValue::NumericDataValue(std::size_t c, double v)
: cnt(c), numeric_val(v){
}

const std::string NumericDataValue::get_representation() const {
  std::ostringstream s;
  s << "NumericDataValue: count = " << cnt
    << " sum = " << numeric_val
    << " avg = " << (numeric_val / cnt);
  return s.str();
}

void NumericDataValue::accumulate(const NumericDataValue & ndv){
  this->cnt += ndv.cnt;
  this->numeric_val += ndv.numeric_val;
}

void NumericDataValue::subtract(const NumericDataValue & ndv){
  this->cnt -= ndv.cnt;
  this->numeric_val -= ndv.numeric_val;
}

void NumericDataValue::accumulate(std::size_t c, double v){
  this->cnt += c;
  this->numeric_val += v;
}

NumericDataValueFactory::NumericDataValueFactory(const std::vector<double> & v)
: weight_vec(v), total_val(NumericDataValue::zero){
  std::size_t i;
  for(i = 0; i < v.size(); i++)
    total_val.accumulate(1, weight_vec[i]);
}

NumericDataValue NumericDataValueFactory::create_datavalue(std::size_t id) const
{
  NumericDataValue nv(1, weight_vec[id]);
  return nv;
}

double NDVICVScore::score(const NumericDataValue & match, const NumericDataValue & total_val) const {
  // ICV for now.
  NumericDataValue notmatch(total_val);
  notmatch.subtract(match);
  if(match.get_cnt() && notmatch.get_cnt()){
    return (
	    (match.get_val() * match.get_val() / match.get_cnt())
	    + (notmatch.get_val() * notmatch.get_val() / notmatch.get_cnt())
	    );
  } else {
    return 0.0;
  }
}

double NDVRankSumTestScore::score(const NumericDataValue & match, const NumericDataValue & total_val) const {
  // Wilcoxon Rank sum test statistic.
  std::size_t n1, n2;
  n1 = match.get_cnt();
  n2 = total_val.get_cnt() - n1;
  return(fabs(Scores::ranksumtest(n1, n2, match.get_val(), vc)));
}

std::string NDVRankSumTestScore::eval_str(const NumericDataValue & match,
					  const NumericDataValue & total_val) const {
  std::ostringstream s;
  std::size_t n1, n2;
  double sum1, score;
  n1 = match.get_cnt();
  n2 = total_val.get_cnt() - n1;
  sum1 = match.get_val();
  score = fabs(Scores::ranksumtest(n1, n2, sum1, vc));
  s << " match = " << n1 << " nomatch = " << n2 
    << " match sum = " << sum1 << " match avg = " << (sum1/n1);
  return s.str();
}
