/*
  Copyright (C) 2004 Hideo Bannai (bannai@ims.u-tokyo.ac.jp)
  
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
*/
#ifndef NUMERICDATAVALUE_H
#define NUMERICDATAVALUE_H

#include <cstddef>
#include <cassert>
#include <vector>

class NumericDataValue {
  std::size_t cnt; 
  double numeric_val;
public:
  NumericDataValue();
  NumericDataValue(std::size_t c, double v);
  void accumulate(const NumericDataValue & ndv);
  void subtract(const NumericDataValue & ndv);
  void accumulate(std::size_t c, double v);
  const std::string get_representation() const;
  ~NumericDataValue();

  // return values
  inline std::size_t get_cnt() const { return cnt; };
  inline double      get_val() const { return numeric_val; };

  static const NumericDataValue zero;
};

class NumericDataValueFactory {
  const std::vector<double> & weight_vec;
  NumericDataValue total_val;
 public:
  /* Construct new NumericDataValueFactory. Need to be careful
   * about lifetime of v, since we currently hold a reference to v */
  NumericDataValueFactory(const std::vector<double> & v);
  NumericDataValue create_datavalue(std::size_t id) const;
  const NumericDataValue & get_total() const { return total_val; };
};

class NDVICVScore {
 public:
  NDVICVScore(){};
  double score(const NumericDataValue & match,
	       const NumericDataValue & total_val) const;
};

class NDVRankSumTestScore {
  const double vc;
 public:
  NDVRankSumTestScore(double variance_correction = 0) : vc(variance_correction){};
  /* assumes:
     match.get_cnt()     - number of matching strings
     total_val.get_cnt() - total number of strings

     match.get_val()     - rank sums
     total_val.get_cnt() - total of rank sums (=total_val.get_cnt() * (total_val.get_cnt()+1)/2)
  */
  double score(const NumericDataValue & match,
	       const NumericDataValue & total_val) const;

  std::string eval_str(const NumericDataValue & match,
		       const NumericDataValue & total_val) const;
};

#endif // NUMERICDATAVALUE_H
