/*
  Copyright (C) 2004 Hideo Bannai (bannai@ims.u-tokyo.ac.jp)
  
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
*/

#ifndef RANGEMINIMUMTABLE_H
#define RANGEMINIMUMTABLE_H

#include <cstddef>
#include <vector>

/** Range minimum queries on std::size_t arrays.
 * \bug This class will work only if std::size_t is >= 32 bits.
 * \todo Use template for this class so that it can be used for arrays of
 * any type.
 */
class RangeMinimumTable{
private:
  	/// minimum positions for each block of length log n.
  	std::vector<std::size_t>          block_min;

  	/// sparse table for block_min.
  	std::vector<std::vector<std::size_t> > sparse;

	/// label for calculating rmq inside each block.
	std::vector<std::size_t>          labels;
	
	/// copy of original array
  	std::vector<std::size_t>          array;
public:
	/*** Create table for efficient range minimum query on an array
	 * of unsigned integers. */
  	RangeMinimumTable(const std::vector<std::size_t> & orig_array);

	/// Return the position k such that it is the minimum in the range of [i,j].
	std::size_t query(std::size_t i, std::size_t j) const;
};

#endif // RANGEMINIMUMTABLE_H

