/*
  Copyright (C) 2004 Hideo Bannai (bannai@ims.u-tokyo.ac.jp)
  
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
*/

#include "StringUtils.h"
#include <iostream>
#include <cctype>
#include <cstddef>
#include <cassert>

std::string StringUtils::substr(const std::string & s, Origin o,
				std::size_t s_start, std::size_t s_len){
  std::string res;
  switch(o){
  case BEGIN:
    // assert(s_start < s.length());
    // assert(s_start + s_end < s.length());
    return s.substr(s_start, s_len);
  case END:
    // assert(s_start < s.length());
    // assert(s_start <= s_len);
    return s.substr(s.length() - s_start - 1, s_len);
  default:
    assert(0);
  }
  return res;
}

std::vector<std::string> StringUtils::split(const std::string & s,
                                            const std::string & delimiter){
  std::vector<std::string> newrow;
  std::size_t bpos, epos;
  bpos = 0;
  while(true){
    epos = s.find_first_of(delimiter, bpos);
    if(epos == std::string::npos){
      newrow.push_back(s.substr(bpos, s.length() - bpos + 1));
      break;
    }
    newrow.push_back(s.substr(bpos, epos - bpos));
    bpos = epos + 1;
  }
  return(newrow);
}


std::vector<std::string> StringUtils::split(const std::string & s,
                                            char delimiter){
  std::vector<std::string> newrow;
  std::size_t bpos, epos;
  bpos = 0;
  while(true){
    epos = s.find_first_of(delimiter, bpos);
    if(epos == std::string::npos){
      newrow.push_back(s.substr(bpos, s.length() - bpos + 1));
      break;
    }
    newrow.push_back(s.substr(bpos, epos - bpos));
    bpos = epos + 1;
  }
  return(newrow);
}

void StringUtils::strip_d(std::string & s){
  std::size_t fwdpos, bckpos;
  for(fwdpos = 0; fwdpos < s.length(); fwdpos++){
    if(!std::isspace(s[fwdpos])) break;
  }
  for(bckpos = s.length(); bckpos > 0; bckpos--){
    if(!std::isspace(s[bckpos-1])) break;
  }
  s.erase(bckpos);
  s.erase(0, fwdpos);
}

std::string StringUtils::strip(const std::string & s){
  std::size_t fwdpos, bckpos;
  for(fwdpos = 0; fwdpos < s.length(); fwdpos++){
    if(!std::isspace(s[fwdpos])) break;
  }
  for(bckpos = s.length(); bckpos > 0; bckpos--){
    if(!std::isspace(s[bckpos-1])) break;
  }
  return(s.substr(fwdpos, bckpos - fwdpos));
}
