/*
  Copyright (C) 2004 Hideo Bannai (bannai@ims.u-tokyo.ac.jp)
  
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
*/

#ifndef STRINGUTILS_H
#define STRINGUTILS_H

#include <string>
#include <vector>

class StringUtils{
private:
  StringUtils();
public:
  enum Origin { BEGIN, END };

  /** return substring of s with respect to Origin o.
   * If Origin is BEGIN:
   *   return normal substring beginning at s_start with length s_len
   * If Origin is END:
   *   return substring beginning at end - s_start with length s_len
   *   substr(s, END, 0, 1) returns the last character of the string.
   */
  static std::string substr(const std::string & s, Origin o, 
			    std::size_t s_start, std::size_t s_len);
  
  static std::vector<std::string> split(const std::string & s,
					const std::string & delim);
  
  static std::vector<std::string> split(const std::string & s,
					char delim);
  
  /** Destructively remove preceding and trailing white space.
   * Modifies original string */
  static void strip_d(std::string & s);
  
  /** remove preceding and trailing white space.
   * Does not modify original string, and returns string */
  static std::string strip(const std::string & s);
};

#endif // STRINGUTILS_H
