/*
  Copyright (C) 2004 Hideo Bannai (bannai@ims.u-tokyo.ac.jp)
  
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
*/

#ifndef TAGGEDNUMERICDATAVALUE_H
#define TAGGEDNUMERICDATAVALUE_H

#include <cstddef>
#include <vector>
#include <string>

class TaggedNumericDataValue{
  std::size_t tagged_cnt, untagged_cnt;
  double tagged_val, untagged_val;
public:
  TaggedNumericDataValue();
  TaggedNumericDataValue(std::size_t tc, double tv, std::size_t uc, double uv);
  void accumulate(const TaggedNumericDataValue & tndv);
  void subtract(const TaggedNumericDataValue & tndv);
  void accumulate(std::size_t tc, double tv, std::size_t uc, double uv);

  const std::string get_representation() const;
  ~TaggedNumericDataValue();
  
  inline std::size_t get_tagged_cnt() const { return tagged_cnt; };
  inline std::size_t get_untagged_cnt() const { return untagged_cnt; };
  inline double get_tagged_val() const { return tagged_val; };
  inline double get_untagged_val() const { return untagged_val; };

  static const TaggedNumericDataValue zero;

};

class TaggedNumericDataValueFactory {
  const std::vector<double> weight_vec;
  const std::vector<bool>   tag_vec;
  TaggedNumericDataValue total_val;
public:
  /* Construct new TaggedNumericDataValueFactory. */
  TaggedNumericDataValueFactory(const std::vector<double> & v,
                                const std::vector<bool> & tagv);
  TaggedNumericDataValue create_datavalue(std::size_t id) const;
  const TaggedNumericDataValue & get_total() const { return total_val; }
};

class TNDVICVScore {
  std::size_t combo;
 public:
  TNDVICVScore(std::size_t combos);
  double score(const TaggedNumericDataValue & match,
	       const TaggedNumericDataValue & total_val) const;
  std::string eval_str(const TaggedNumericDataValue & match,
		       const TaggedNumericDataValue & total_val) const;
};

class TNDVRankSumTestScore {
  std::size_t combo;
  double vc;
 public:
  /* a combination of a pattern is expressed by a 
     4 bit integer, 0 to 15.
     p1  p2 | output
     ---------------
     0   0  |  bit 1  1 : (not p1) and (not p2)
     0   1  |  bit 2  2 : (not p1) and p2
     1   0  |  bit 3  4 : p1 and (not p2)
     1   1  |  bit 4  8 : p1 and p2 */
  TNDVRankSumTestScore(std::size_t c, double variance_correction = 0.0){ combo = c; vc = variance_correction; }
  double score(const TaggedNumericDataValue & match,
	       const TaggedNumericDataValue & total_val) const;

  std::string eval_str(const TaggedNumericDataValue & match,
		       const TaggedNumericDataValue & total_val) const;
};

#endif // TAGGEDNUMERICDATAVALUE_H
