/*
  Copyright (C) 2004 Hideo Bannai (bannai@ims.u-tokyo.ac.jp)
  
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
*/

#include "Utils.h"
#include <algorithm>

namespace Utils {
  double to_ranks_of(const std::vector<double> & wv,
		     std::vector<double> & result){
    std::size_t i,j;
    std::vector<std::size_t> ranking_vec(wv.size());
    result.resize(wv.size());
    for(i = 0; i < wv.size(); i++) ranking_vec[i] = i;
    std::sort(ranking_vec.begin(), ranking_vec.end(), ArrayCmp(wv));
    for(i = 0; i < wv.size(); i++) result[ranking_vec[i]] = i+1;


    std::size_t tiecnt = 1;
    double variance_correction = 0;

    for(i = 1; i <= wv.size(); i++){
      if(i < wv.size() && (wv[ranking_vec[i]] <= wv[ranking_vec[i-1]])){
	tiecnt++;
      } else {
	if(tiecnt > 1){
	  /* assign tie scores */
	  for(j = i-tiecnt; j < i; j++){
	    result[ranking_vec[j]] = (((double) i + i - tiecnt + 1) / 2);
	  }
	  double dtc = (double) tiecnt;
	  variance_correction +=  dtc * (dtc * dtc - 1.0) / 12.0;
	  tiecnt = 1;
	}
      }
    }
    return variance_correction;
  }
};
