/*
  Copyright (C) 2004 Hideo Bannai (bannai@ims.u-tokyo.ac.jp)
  
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
*/

#ifndef __UTILS_H__
#define __UTILS_H__

#include <vector>
#include <functional>

namespace Utils {

  /** compare operator weight vector. Given two positions
      x, y of a double vector w, return whether wv[x] < wv[y] */
  struct ArrayCmp : public std::binary_function<double,double,bool> {
    const std::vector<double> & wv;
    ArrayCmp(const std::vector<double> & weightv) : wv(weightv){};
    bool operator()(const std::size_t x, const std::size_t y) const {
      if(wv[x] == wv[y]){
	return x < y;
      } else {
	return wv[x] < wv[y];
      }
    }
  };

  /** convert a vector of doubles to a vector of their ranks. 
      wv is unchanged.
      result[i] is converted to the rank of the ith weight.
      Return correction factor of variance of rank sum. */
  double to_ranks_of(const std::vector<double> & wv,
		     std::vector<double> & result);
};
#endif//__UTILS_H__
