/*
  Copyright (C) 2004 Hideo Bannai (bannai@ims.u-tokyo.ac.jp)
  
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
*/

#include <iostream>
#include <boost/test/unit_test.hpp>
#include "../DBTable.h"

using namespace std;
using namespace boost::unit_test_framework;

void DBTable_read_test () {
  vector<string> tmpvec;
  size_t row;
  LineProcessor lp("\t,");
  DBTable tbl(string("DBTable_test.txt"), lp);
  for(row = 0; row < tbl.get_row_count(); row++){
    const vector<string> & v = tbl.get_row(row);
    for(size_t i = 0; i < v.size(); i++){
      cout << "Row: " << row << " Col: " << i << " | " << v[i] << endl;
    }
  }

  tbl.get_col(3, tmpvec);

  BOOST_CHECK_EQUAL( tmpvec.size(), tbl.get_row_count() );

  cout << "Output column 3 ----" << endl;
  for(row = 0; row < tmpvec.size(); row++){
    cout << "Row: " << row << "\t" << tmpvec[row] << endl;
  }

  cout << "Filter with column 3 ---" << endl;
  vector<size_t> filt(1);
  filt[0] = 3;
  tbl.filter(filt);
  for(row = 0; row < tbl.get_row_count(); row++){
    const vector<string> & v = tbl.get_row(row);
    for(size_t i = 0; i < v.size(); i++){
      cout << "Row: " << row << " Col: " << i << " | " << v[i] << endl;
    }
  }
  return;
}

test_suite* init_unit_test_suite( int argc, char* argv[] )
{
  test_suite* test = BOOST_TEST_SUITE( "DBTable test" );
  test->add(BOOST_TEST_CASE(DBTable_read_test));
  return test;
}

