/*
  Copyright (C) 2004 Hideo Bannai (bannai@ims.u-tokyo.ac.jp)
  
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
*/

#include <iostream>
#include <boost/test/unit_test.hpp>
#include "../NumericDataValue.h"
#include "../SAColorSetSize.h"

using namespace std;
using namespace boost::unit_test_framework;

typedef SAColorSetSizeResult<NumericDataValue> resulttype;
void NumericDataValue_test(){
  size_t i;
  string s = "mississippi";
  vector<double> weightvec(1);
  SuffixArray sa(s);
  NumericDataValueFactory fctry(weightvec);
  vector<resulttype> results;
  vector<resulttype>::iterator it;
  NDVICVScore icvscore;

  results = SAColorSetSize<NumericDataValue,NumericDataValueFactory,NDVICVScore>::solve(sa, fctry, icvscore, 10);
  for(i = 0; i < results.size(); i++){
    cout << i << ": score: " << results[i].score
	 << " (" << results[i].node.l
	 << "," << results[i].node.h
	 << ") "
	 << "'" << sa.get_substr(results[i].node) << "'"
	 << " [" << results[i].dv.get_representation() << "]" << endl;
  }

  results = SAColorSetSize<NumericDataValue,NumericDataValueFactory,NDVICVScore>::solve(sa, fctry, icvscore, 10, false);
  for(i = 0; i < results.size(); i++){
    cout << i << ": score: " << results[i].score
	 << " (" << results[i].node.l
	 << "," << results[i].node.h
	 << ") "
	 << "'" << sa.get_substr(results[i].node) << "'"
	 << " [" << results[i].dv.get_representation() << "]" << endl;
  }
}

void NumericDataValue_multi_test(){
  cout << "NumericDataValue_multi_test()" << endl;
  size_t i;
  string s1("mississippi");
  string s2("mmmmissppi");
  vector<string> strvec;
  vector<double> weightvec(2);
  vector<resulttype> results;
  vector<resulttype>::iterator it;
  strvec.push_back(s1);
  strvec.push_back(s2);
  weightvec[0] = 1.0;
  weightvec[1] = -1.0;
  SuffixArray sa(strvec);

  NumericDataValueFactory fctry(weightvec);
  NDVICVScore icvscore;
  results = SAColorSetSize<NumericDataValue,NumericDataValueFactory,NDVICVScore>::solve(sa, fctry, icvscore, 10, false);
  for(i = 0; i < results.size(); i++){
    cout << i << ": score: " << results[i].score
	 << " (" << results[i].node.l
	 << "," << results[i].node.h
	 << ")\t"
	 << "[" << results[i].dv.get_representation() << "]" 
	 << "\t'" << sa.get_substr(results[i].node) << "'"
	 << endl;
  }
}

test_suite* init_unit_test_suite( int argc, char* argv[] )
{
  test_suite* test = BOOST_TEST_SUITE( "NumericDataValue test" );
  test->add(BOOST_TEST_CASE(NumericDataValue_test));
  test->add(BOOST_TEST_CASE(NumericDataValue_multi_test));
  return test;
}
