/*
  Copyright (C) 2004 Hideo Bannai (bannai@ims.u-tokyo.ac.jp)
  
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
*/

//
// $Id: rmq_test.cpp 146 2004-06-14 14:23:16Z bannai $
// test for rmq
//
#include "../RangeMinimumTable.h"
#include <iostream>
#include <boost/test/unit_test.hpp>
#include <boost/random.hpp>

using namespace std;
using namespace boost::unit_test_framework;

/// Test range minimum query
void rmq_test (){
 	std::size_t i, j, size;
	std::size_t sizes[9] = { 1, 4, 8, 16, 31, 32, 33, 65, 200 };
	for(size = 0; size < 9; size++){
	 	vector<std::size_t> array(sizes[size]);
	  	boost::mt19937 rng;
	  	boost::uniform_int<> uni_dist(-100,100);
		boost::variate_generator<boost::mt19937,boost::uniform_int<> > uni(rng,uni_dist);
		cout << "Checking constant time range minimum query for size: "
		     << array.size() << endl;
	  	for(i = 0; i < array.size(); i++)
    		array[i] = uni();
	    	
	  	RangeMinimumTable rmtable(array);
	  	for(i = 0; i < array.size(); i++){
	    	for(j = i; j < array.size(); j++){
	      		std::size_t p, pos1, pos2;
	      		pos1 = rmtable.query(i,j);
	
				// naive rmq ///
		      	for(pos2 = p = i; p <= j; p++){
					if(array[p] < array[pos2]) pos2 = p;
		      	}
			if(array[pos1] != array[pos2]){
			  cout << "pos1 = " << pos1 << ", pos2 = " << pos2 << endl;
			}
		      	BOOST_CHECK_EQUAL(array[pos1], array[pos2]);
		    }
		  }
	}
}

test_suite* init_unit_test_suite( int argc, char* argv[] )
{
  test_suite* test = BOOST_TEST_SUITE( "RangeMinimumTable test" );
  test->add( BOOST_TEST_CASE( &rmq_test ) );
  return test;
}
