/*
  Copyright (C) 2004 Hideo Bannai (bannai@ims.u-tokyo.ac.jp)
  
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
*/

#include <iostream>
#include <boost/test/unit_test.hpp>
#include <string>
#include "../StringUtils.h"

using namespace std;
using namespace boost::unit_test_framework;

void StringUtils_substr_test(){
  string s("this is my string");
  BOOST_CHECK_EQUAL(string("is is"), StringUtils::substr(s, StringUtils::BEGIN, 2, 5));
  BOOST_CHECK_EQUAL(string("is is"), StringUtils::substr(s, StringUtils::END, 14, 5));
}

void StringUtils_split_test(){
  vector<string> res;
  char * myres[4] = { "a", "tab", "separated", "string!" };
  res = StringUtils::split(string("a\ttab\tseparated\tstring!"),
			   string("\t"));
  BOOST_CHECK_EQUAL(res.size(), static_cast<size_t>(4));
  for(int i = 0; i < 4; i++){
    BOOST_CHECK_EQUAL(res[i],string(myres[i]));
  }

  char * myres2[4] = { "a", "tab", "", "string!" };
  res = StringUtils::split(string("a\ttab\t\tstring!"),
			   string("\t"));
  BOOST_CHECK_EQUAL(res.size(), static_cast<size_t>(4));
  for(int i = 0; i < 4; i++){
    BOOST_CHECK_EQUAL(res[i],string(myres2[i]));
  }
  return;
}

void StringUtils_strip_test(){
  string s;
  s = 
    StringUtils::strip("  \t string with leading and trailing white space  ");
  BOOST_CHECK_EQUAL(s, string("string with leading and trailing white space"));
  BOOST_CHECK_EQUAL(StringUtils::strip("    "), string(""));
  BOOST_CHECK_EQUAL(StringUtils::strip("no whitespace on ends"),
		    string("no whitespace on ends"));
  return;
}

void StringUtils_strip_d_test(){
  string s1("  \t string with leading and trailing white space  ");
  string s2("  \t \t");
  string s3("no whitespace on ends");
  StringUtils::strip_d(s1);
  StringUtils::strip_d(s2);
  BOOST_CHECK_EQUAL(s1, string("string with leading and trailing white space"));
  BOOST_CHECK_EQUAL(s2, string(""));
  BOOST_CHECK_EQUAL(s3, string("no whitespace on ends"));
}

test_suite* init_unit_test_suite( int argc, char* argv[] )
{
  test_suite* test = BOOST_TEST_SUITE( "StringUtils test" );
  test->add(BOOST_TEST_CASE(StringUtils_substr_test));
  test->add(BOOST_TEST_CASE(StringUtils_split_test));
  test->add(BOOST_TEST_CASE(StringUtils_strip_test));
  test->add(BOOST_TEST_CASE(StringUtils_strip_d_test));
  return test;
}
