/*
  Copyright (C) 2004 Hideo Bannai (bannai@ims.u-tokyo.ac.jp)
  
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
*/

#include "../SuffixArray.h"
#include <iostream>
#include <boost/test/unit_test.hpp>

using namespace std;
using namespace boost::unit_test_framework;

// BOOST_CHECK( v.empty() );
// BOOST_CHECK_EQUAL( v.size(), 0 );

unsigned int naive_lcp(const char * p, const char * q){
  unsigned int res = 0;
  while(*p == *q && (*p != '\0')){
    res++; p++; q++;
  }
  return res;
}


/// Test contents of suffix array
void sa_test (){
  unsigned int i;
  string s("mississippi");
  SuffixArray sa(s);
  // 11     
  // 10	    i
  // 7	    ippi
  // 4      issippi
  // 1	    ississippi
  // 0	    mississippi
  // 9	    pi
  // 8	    ppi
  // 6	    sippi
  // 3	    sissippi
  // 5	    ssippi
  // 2	    ssissippi
  unsigned int sa_test[12]   = { 11,10, 7, 4, 1, 0, 9, 8, 6, 3, 5, 2 };
  unsigned int rank_test[12] = {  5, 4,11, 9, 3,10, 8, 2, 7, 6, 1, 0 };
  unsigned int lcp_test[12]  = {  0, 0, 1, 1, 4, 0, 0, 1, 0, 2, 1, 3 };
  unsigned int id_test[12]   = {  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 };
  cout << "Checking SuffixArray ----------" << endl;
  BOOST_CHECK_EQUAL(sa.size(), (unsigned int) 12);
  for(i = 0; i < sa.size(); i++){
	//    cout << "Position: " << i << endl;
	//    cout << "Suffix  : " << i << "'" << (s.c_str() + sa.get_suffix(i)) << "'" << endl;
    BOOST_CHECK_EQUAL(sa.get_suffix(i), sa_test[i]);
    BOOST_CHECK_EQUAL(sa.get_rank(i), rank_test[i]);
    BOOST_CHECK_EQUAL(sa.get_lcp(i), lcp_test[i]);
    BOOST_CHECK_EQUAL(sa.get_id(i), id_test[i]);
  }
  cout << endl;
}

void sa_multi_test (){
  cout << "Checking multi suffixarray ----------" << endl;
  unsigned int i;
  string s1("mississippi");
  string s2("mmmmissppi");
  vector<string> strvec;
  strvec.push_back(s1);
  strvec.push_back(s2);
  SuffixArray sa(strvec);
  for(i = 0; i < sa.size(); i++){
    cout << i << "\t" << sa.get_suffix(i) << "\t"
	 << sa.get_rank(i) << "\t"
	 << sa.get_lcp(i) << "\t"
         << sa.get_id(i) << "\t'"
         << (sa.get_string().substr(sa.get_suffix(i))) << "'" << endl;
    if(i > 0){
      const char * s1, * s2;
      s1 = sa.get_string().c_str() + sa.get_suffix(i-1);
      s2 = sa.get_string().c_str() + sa.get_suffix(i);
      BOOST_CHECK_EQUAL(sa.get_lcp(i), naive_lcp(s1, s2));
    }
  }
  cout << endl;
}

void sa_iterator_test(){
  string s("mississippi");
  SuffixArray sa(s);
  SuffixArrayIterator it(sa);
  SASubstring l;
  int cnt = 0;
  do {
    cnt++;
    l = it.get_next();
    cout << "(" << l.l << "," << l.h << ")\t'" << sa.get_substr(l) << "'" << endl;
  } while (l.l || l.h);
  BOOST_CHECK_EQUAL(cnt,19);
  return;
}

test_suite* init_unit_test_suite( int argc, char* argv[] )
{
  test_suite* test = BOOST_TEST_SUITE( "strlib::SuffixArray test" );
  test->add( BOOST_TEST_CASE( &sa_test ) );
  test->add( BOOST_TEST_CASE( &sa_multi_test ) );
  test->add( BOOST_TEST_CASE( &sa_iterator_test ) );
  return test;
}
