/*
  Copyright (C) 2004 Hideo Bannai (bannai@ims.u-tokyo.ac.jp)
  
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
*/

#include <iostream>
#include <boost/test/unit_test.hpp>
#include "../TaggedNumericDataValue.h"
#include "../SAColorSetSize.h"

using namespace std;
using namespace boost::unit_test_framework;

typedef SAColorSetSizeResult<TaggedNumericDataValue> resulttype;
  
void TaggedNumericDataValue_test(){
  string s1("mississippi");
  string s2("mmmmissppi");
  vector<string> strvec;
  strvec.push_back(s1);
  strvec.push_back(s2);
  SuffixArray sa(strvec);
  vector<double> weightvec(strvec.size());  
  vector<bool> tagvec(strvec.size());

  for(size_t i = 0; i < sa.size(); i++){
    cout << i << "\t" << sa.get_suffix(i) << "\t"
         << sa.get_id(i) << "\t'"
         << (sa.get_string().substr(sa.get_suffix(i))) << "'" << endl;
  }

  weightvec[0] =  1.0; tagvec[0] = true;
  weightvec[1] = -1.0; tagvec[1] = true;

  TaggedNumericDataValueFactory fctry(weightvec, tagvec);
  TNDVRankSumTestScore ranksumtest(8); /* p AND q */
  vector<resulttype> results;
  results = SAColorSetSize<TaggedNumericDataValue,TaggedNumericDataValueFactory,TNDVRankSumTestScore>::solve(sa, fctry, ranksumtest, 50, false);
  for(size_t i = 0; i < results.size(); i++){
    cout << i << ": score: " << results[i].score
	 << " (" << results[i].node.l
	 << "," << results[i].node.h
	 << ")\t"
	 << "[" << results[i].dv.get_representation() << "]" 
	 << "\t'" << sa.get_substr(results[i].node) << "'"
	 << endl;
  }
}

test_suite* init_unit_test_suite( int argc, char* argv[] )
{
  test_suite* test = BOOST_TEST_SUITE( "TaggedNumericValue test" );
  test->add(BOOST_TEST_CASE(TaggedNumericDataValue_test));
  return test;
}
