#!/usr/bin/ruby
#
# Requires BioRuby (http://bioruby.org/)
#
# Usage:
#   assign_sequence.rb fastasequencefile
#
# Reads from standard input and adds "tab + sequence" of each line
# that contains a sequence name in the fasta file.
#
# Copyright (C) 2004 Hideo Bannai (bannai@ims.u-tokyo.ac.jp)
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#  
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

require 'bio'

ff = Bio::FastaFormat.open(ARGV.shift)
db = {}

ff.each { |entry|
  ar = entry.definition.split
  db[ar[0]] = entry
}

$stdin.each_line { |line|
  line.chomp!
  cnt = 0
  db.each do |name,entry|
    regex = /(#{name})(\s|[;|]|$)/
    if regex =~ line then
      if cnt > 0 then
        puts "WARNING: two different definitions of same name: #{ar[0]}"
      end
      line = line + "\t" + entry.definition + "\t" + entry.seq
      cnt += 1
    end
  end
  puts line
}
